/**
 * Native Bridge to JS
 * @param method string Method name
 * @param params array Array of Parameters
 * @param jsCallback JavaScript callback
 */
function comNativeBridgePlayer(methodName, paramArray, jSCallback) {
    if(typeof("NativeBridge")!="undefined") { // obj-c 2 JS implementation
        NativeBridge.call(methodName, paramArray, jSCallback);
    }
}

function testCallback(val) {
}

function testPlayer() {
    return "testPlayer";
}

var playerSetupped = false;

var Logger = {

    // 0 | 1
    level : 0,
    setLevel : function(l) {this.level=l;},
    getLevel : function() { return this.level; },
    log: function(msg) {
        if(this.getLevel()) {
            console.log(msg);
        }
    },
    init : function() {
        this.setLevel(0);
    }
    
}

var Player  = {
    
    position : 0.0,
    playing : false,
    isAvailable: false,
    latestLyricsIDLoaded: -1,
    lastLoadParams: null,
    lastToggleSynched: null,
    
    getPosition : function() {  return this.position; },
    setPosition : function(pos) {this.position=pos; },
    isPlaying : function(val) {  this.playing=val; },
    setIsAvailable : function(val) {  this.isAvailable=val; },
    
    init : function() {
        playerSetupped = true;
        var instance = this;
        //Adding synchatore HTML
        //HTMLAddSyncedLyrics();
        syncatore.setup({
                        //containerId: "synctool",
                        intro: "...",
                        outro: "(end)",
                        compact: true,
                        compactSyncing: true,
                        translate3d: false
                        });
        syncatore.app.platform = 'android';
        syncatore.app.transition = false;
        syncatore.app.allowManualScrolling = true;
        syncatore.app.clickToSeek = false;
		
        // Event that triggers as soon as the user syncs the first line.
        // It will send the id of the track (subtitle).
        syncatore.app.bind('onSyncStarted', function(id) {
                           AndroidNativeBridgeMXM.onSyncStarted(id);
                           });
        
        // Event that triggers as soon as the user has finished syncing all the lines and reached the last line.
        // The data object sent has two properties: id and lyrics (the JSON object of the synced lyrics)
        
        syncatore.app.bind('onSyncFinished', function(data) {
          AndroidNativeBridgeMXM.onSyncFinished(JSON.stringify(data.lyrics));
          return;
        });
        
        // Event that triggers as soon as the media playback is paused and the user is in sync mode.
        // It will send the id of the track (subtitle).
        
        syncatore.app.bind('onSyncPaused', function(id) {
                           AndroidNativeBridgeMXM.onSyncPaused(id);
                           });
        
        // Event that triggers as soon as the user exists the sync mode or the playback is stopped.
        // It will send the id of the track (subtitle).
        
        syncatore.app.bind('onSyncStopped', function(id) {
                           AndroidNativeBridgeMXM.onSyncStopped(id);
                           });
        
        syncatore.app.bind('onCurrentLineChanged', function(line) {
                           var params= new Array();
                           params.push(line);
                           comNativeBridgePlayer("onCurrentLineChanged",params);
                           });
        
        syncatore.app.bind('onViewRendered', function(line) {
                           var params= new Array();
                           params.push(line);
                           
                           comNativeBridgePlayer("onViewRendered",params);
                           
                           });
    
        // For connecting the syncatore to a media player the following methods need to be overridden
        
        syncatore.player.play = function() {
            AndroidNativeBridgeMXM.playerEventsPlay();
        };
        
        syncatore.player.isPlaying = function() {
            return instance.playing;
        };
        
        syncatore.player.getPosition = function() {
            var pos = AndroidNativeBridgeMXM.getCurrentPosition();
            return pos;
        };
        
        syncatore.player.seek = function(seconds) {
            AndroidNativeBridgeMXM.playerEventsSeek(seconds);
        };

        syncatore.player.isAvailable = function() {
            return instance.isAvailable;
        };
        
    },
    
    stateChange : function(state) {
        
        this.isPlaying((state=="playing"));

        if(typeof(syncatore)!="undefined") {

            syncatore.player.stateChanged(state);
        }
        
    },

    getSyncData : function() {
        return JSON.stringify(syncatore.track.lyrics.toJSON());
    },
    
    seek : function(seconds) {
        syncatore.player.positionChanged(seconds);
    },

    /**
     * Save sync tool from the first line
     * @return
     */
    saveSync : function() {
        syncatore.save();
    },
    
    /**
     * Reset sync tool from the first line
     * @return
     */
    resetSync : function() {
        syncatore.app.resetSync();
    },

    /**
     * Manually set clickToSeek function
     * @return
     */
    clickToSeek : function(enable) {
        syncatore.app.clickToSeek = enable;
    },

    /**
     * Get current selected line by Syncatore
     * @return
     */
    getCurrentLineText : function() {
      var line = syncatore.app.getCurrentLine().text;
        AndroidNativeBridgeMXM.updateCurrentLineText(line);
    },

    /**
     * Get current selected line by Syncatore
     * @return
     */
    getPrevLineText : function() {
      var line = syncatore.app.getPrevLine().text;
        AndroidNativeBridgeMXM.updatePrevLineText(line);
    },

    /**
     * Get current selected line by Syncatore
     * @return
     */
    getNextLineText : function() {
      var line = syncatore.app.getNextLine().text;
        AndroidNativeBridgeMXM.updateNextLineText(line);
    },
    
    toggleCompact : function() {
        syncatore.app.toggleCompact();
    },

    toggleCompactParam : function( value ) {
        
       syncatore.app.toggleCompact(value);
    },

    setMusicIDPosition : function(startPosition, clockTimeNative) {
      startPosition = calculateClockDiff(startPosition, clockTimeNative);
      syncatore.app.setStartPosition(startPosition);
    },

    toggleSynced : function( toggleSynced ) {
      //syncatore.app.toggleSynced(toggleSynced);

      if( this.lastLoadParams == null || (this.lastToggleSynched != null && this.lastToggleSynched == toggleSynced) || (this.lastToggleSynched == null && !toggleSynced) ) {
        
      } else {
        Player.setIsAvailable(!toggleSynced);
        this.lastToggleSynched = toggleSynced;
        this.load(this.lastLoadParams.lyricsID, this.lastLoadParams.id, this.lastLoadParams.data, this.lastLoadParams.copyright, this.lastLoadParams.startMode, this.lastLoadParams.autoPlay, this.lastLoadParams.startPosition, this.lastLoadParams.clockTimeNative, true);
      }
    },

    load : function(lyricsID, id, data, copyright, startMode, autoPlay, startPosition, clockTimeNative, forceReload) {
        console.log(data);
        this.lastLoadParams = {
          'lyricsID': lyricsID,
          'id': id,
          'data': data,
          'copyright': copyright,
          'startMode': startMode,
          'autoPlay': autoPlay,
          'startPosition': startPosition, 
          'clockTimeNative': clockTimeNative
        }

        if ( this.latestLyricsIDLoaded != -1 && this.latestLyricsIDLoaded == lyricsID && !forceReload ) {
          if( autoPlay ) {
            startPosition = calculateClockDiff(startPosition, clockTimeNative);
            this.lastLoadParams.startPosition = startPosition;
            syncatore.app.setStartPosition(startPosition);
          }


            var params= new Array();
            comNativeBridgePlayer("onViewRendered",params);
            return;
        }

        this.latestLyricsIDLoaded = lyricsID;
        var jsonData;
        var status = "OK";
        
        try {
            jsonData = JSON.parse(unescape(data));
        } catch(ex) {

            status = "Error "+ex;
            jsonData=unescape(data);
        }
        
        try {
            
            if(typeof(syncatore)!="undefined") {
                //HTMLAddSyncedLyrics();
                var loadParams = {
                  'id': id,
                  'lyrics': jsonData,
                  'copyright': copyright,
                  'compact': startMode,
                  'autoPlay': autoPlay,
                  'syncOnLoad' : true,
                }
                if ( autoPlay ) {
                    loadParams.startPosition = calculateClockDiff(startPosition, clockTimeNative);
                    this.lastLoadParams.startPosition = startPosition;
                }
                var toStamp = {
                  'id': id,
                  'compact': startMode,
                  'autoPlay': autoPlay,
                }
                if ( autoPlay ) {
                    toStamp.startPosition = startPosition;
                }
                syncatore.load(loadParams);
                /*if(!syncatore.app.syncFinished()) {
                    syncatore.app.toggleSyncing();
                }*/
                //setWebViewHeight( $(window).height() );
                
                //Logger.log(  syncatore.track.toJSON() );
                
                status = "Loaded";//JSON.stringify(jsonData);
                
                comNativeBridgePlayer("PlayerEventsInit");
            
            }
            else {
                status = "Not Found";//JSON.stringify(jsonData);
            }
        
        } catch(ex) {
            status = "Error "+ex;
        }
        
        Logger.log("Player status "+status);
        
         
    },

    /**
     * Start sync
     * @return
     */
    startSyncing : function() {
        syncatore.app.startSyncing();
    },
    
    /**
     * Stop sync
     * @return
     */
    stopSyncing : function() {
        if( typeof( sync )!='undefined') {
            syncatore.app.stopSyncing();
        }
    },
    
    /**
     * Sync current line and go to the next one
     * @return
     */
    syncNext : function() {
        syncatore.app.sync();
    },
    
    /**
     * Cancel last synced line
     * @return
     */
    undoSync : function() {
        syncatore.app.undo();
    },

    /**
     * Reload the latest
     * @return
     */
    reload : function() {
        syncatore.reload();
    },
    
    /**
     * Toggle sync on/off
     * @return
     */
    toggleSyncing : function() {
        syncatore.app.toggleSyncing();
    },
    
    /**
     *
     * @return true if sync is on
     */
    isSyncing : function () {
        return syncatore.app.syncing;
    },

};

function calculateClockDiff(startPosition, nativeClock) {
  var now = (new Date()).getTime();
  var clock = Math.abs(now - nativeClock) / 1000.0;
  return Math.round((startPosition + clock) * Math.pow(10, 2)) / Math.pow(10, 2);
}

$(document).ready(function() {
                  
    Logger.init();
    if ( !playerSetupped ) 
        Player.init();
                  
    comNativeBridgePlayer("documentIsReady");
    
});